#include "config.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "def.h"
#include "usb.h"

#include "usbi2c.h"

uint8_t i2cbuffer[32];
uint8_t i2cblen, i2cbhead;

#define F_I2C 50000

#include "i2c.c"


uint8_t usb_in(byte_t* data, byte_t len)
{
   // return the data requested by the usb transfer.
   byte_t i;
   for(i=0;i<len && i2cbhead<i2cblen;i++)
   {
     data[i]= i2cbuffer[i2cbhead];
     i2cbhead++;
   }
   return i;
}

void usb_out(byte_t* data, byte_t len)
{
   byte_t i;
   for(i=0;i<len;i++)
    {
      i2cbuffer[i2cblen]= data[i];
      i2cblen= (i2cblen + 1)%32;
    }
   // use the data specified by the usb transfer
}

void handle_port(uint8_t op);

byte_t usb_setup(byte_t data[8])
{
  uint8_t req;
  uint16_t value, index;
  // the setup packet is structured as follows:
  // offset size meaning
  //  0      1   request type, describes the endpoint direction etc.
  //  1      1   request code. any value is OK, since the above
  //                           field already specifies VENDOR and
  //                           we're free to use any value.
  //  2      2   value    these two fields are generic fields for
  //  4      2   index    identifying stuff. I just use them as data fields
  //  6      2   length of data to follow in the _out() packets
  req= data[1];
  // if there is data to be returned, we can do that within the data[]
  // array, using up to 8 bytes. If there are more, we should return
  // 0xff and usbtiny will call usb_in().
  switch(req)
  {
  case I2C_fill:
    i2cblen= 0;
    return 0;
  case I2C_write:
    value= *(uint16_t*) (data+2);
    if (!i2cblen) data[0]= I2C_ok; 
             else data[0]= i2c_write(value);
    return 1;
  case I2C_write_read:
    value= *(uint16_t*) (data+2);
    index= *(uint16_t*) (data+2);
    if (!i2cblen) data[0]= i2c_read(value,index);
             else data[0]= i2c_write_read(value,index);
    return 1;
  case I2C_read:
    value= *(uint16_t*) (data+2); // device address
    index= *(uint16_t*) (data+4); // read length
    data[0]= i2c_read(value,index);
    return 1;
  case I2C_get:
    i2cbhead= 0;
    return 0xff;
  case PORTOP:
    value= *(uint16_t*) (data+2);
    handle_port( (uint8_t) value );
    return 0;
  case PINOP:
    value= *(uint16_t*) (data+2);
         if (value==0) data[0]= PINB;
    else if (value==1) data[0]= PINC;
    else               data[0]= PIND;
    return 1;
  }
  return 0; 
}


int main()
{
   DDRB= 0;
   DDRC= 0;
   DDRD= 0;
   PORTB= 0;
   PORTC= 0;
   PORTD= 0;
   usb_init();
   i2c_init();
   while(1) usb_poll();
}

