#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "../firmware/usbi2c.h"
#include "usb.h"

uint8_t data[8];
int mode;

#define DS1307 0x68

int read_bits(int B, int b0, int b1)
{
  int i;
  int K,Y,V;
  V= data[B];
  K= 0;
  for(i=b1;i<=b0;i++)
  {
    Y= (V >> i) &1;
    K|= (Y<<(i-b1));
  }
  return K;
}

int write_bits(int V, int B, int b0, int b1)
{
  int i,K;
  K=data[B];
  
  for(i=b1;i<=b0;i++)
  {
    K &= ~(1<<i);
    K |= (V&1) << i;
    V>>= 1;
  }
  data[B]= K;
  return 0xff;
}

int BF(int V, int B0, int b01, int b02, int B1, int b11, int b12)
{
  if (mode=='R')
    return read_bits(B0,b01,b02)*10 + 
           read_bits(B1,b11,b12);
  else
  {
           write_bits(V/10,B0,b01,b02);
    return write_bits(V%10,B1,b11,b12);
  }
}

int SF(int V,int B, int b1, int b2)
{
  if (mode=='R') return read_bits(B,b1,b2);
  else return write_bits(V, B,b1,b2);
}

int F(int V, int B, int b)
{
  if (mode=='R') return read_bits(B,b,b);
            else return write_bits(V, B,b,b);
}

int Dseconds(int V) { return BF(V,0,6,4, 0,3,0); }
int Dminutes(int V) { return BF(V,1,6,4, 1,3,0); }
int Dhmode(int V)   { return F(V, 2,6); }
int Dampm(int V)    { return F(V, 2,5); }
int D12hour(int V)  { return BF(V, 2, 4,4, 2, 3, 0); }
int D24hour(int V)  { return BF(V, 2, 5,4, 2, 3, 0); }
int Dday(int V)     { return SF(V, 3, 2, 0); }
int Ddate(int V)    { return BF(V, 4, 5, 4, 4, 3, 0); }
int Dmonth(int V)   { return BF(V, 5, 4, 4, 5, 3, 0); }
int Dyear(int V)    { return BF(V, 6, 7, 4, 6, 3, 0); }
int Dout(int V)     { return F(V, 7,7); }
int Dsqwe(int V)    { return F(V, 7,4); }
int Drs1(int V)     { return F(V, 7,1); }
int Drs0(int V)     { return F(V, 7,0); }
int Dch(int V)      { return F(V, 0,7); }

static struct
{
  char *name; int (*func)(int);
} fields[]= {
  {"ch", Dch }, {"seconds", Dseconds}, {"minutes", Dminutes},
  {"hmode", Dhmode},
  {"ampm", Dampm},
  {"12hours", D12hour },
  {"24hours", D24hour },
  {"day", Dday },
  { "month", Dmonth },
  { "year", Dyear},
  { "out", Dout},
  { "sqwe", Dsqwe },
  { "rs1", Drs1 },
  { "rs0", Drs0 },
  { NULL, NULL }
};

static void prt_data()
{
  int i;
  mode= 'R';
  printf("data start--\n");
  for(i=0;fields[i].name;i++)
   printf("%20s: %d\n", fields[i].name, (*fields[i].func)(0));
  printf("data end--\n");
}

static int get_data()
{
  int R,i;
  R= 0;
  mode= 'W';
  for(i=0;fields[i].name;i++)
  {
    printf("%10s? >",fields[i].name);
    fflush(0);
    char g[2000];
    if (!fgets(g,sizeof(g),stdin)) break;
    g[sizeof(g)-1]= 0;
    int j;
    for(j=0;g[j];j++) if (g[j]=='\n' || g[j]=='\r') g[j--]= 0;
    if (!g[0]) continue;
printf("writing integer %d\n",atoi(g));
    (*fields[i].func)(atoi(g));
    R++;
  }
  return R;
}

static void pbin_data()
{
int i,j;
  printf("data in binary:\n");
  for(i=0;i<8;i++) { printf("%d: ",i);
    for(j=7;j>=0;j--) printf("%c", data[i]&(1<<j) ? '1':'0');
    printf("\n");
  }
  
}

static void read_data()
{
  int r;
  uint8_t D;
   char buf[8];
 printf("starting to read\n");
  D= 0;
  r= usb_write(I2C_fill, 0, 0, &D, 1);
  if (r<0) exit_print_libusb_error("FILL", r);

  r= usb_read(I2C_write_read, DS1307, 8,buf,1);
  if (r<0) exit_print_libusb_error("SENDING WRITE-READ", r);
  if (buf[0]&I2C_error) exit(fprintf(stderr,"I2C error during write-read\n"));

  r= usb_read(I2C_get, 0, 8, data, 8);
  if (r<0) exit_print_libusb_error("READ", r);
printf("read ok: %d\n",r);
pbin_data();
}

static void write_data()
{
  int r;
   char buf[9];
printf("starting to write\n");
  int i;
  for(i=1;i<=5;i++) data[i] &= ~(1<<7); // clear all zero bits
pbin_data();
  for(i=0;i<8;i++) buf[i+1]= data[i];
  buf[0]= 0;
  r= usb_write(I2C_fill, 0, 0, buf, 9);
  if (r<0) exit_print_libusb_error("FILL", r);
  r= usb_read(I2C_write, DS1307,8, buf, 1);
  if (r<0) exit_print_libusb_error("WRITE", r);
  if (buf[0]&I2C_error) exit(fprintf(stderr,"I2C error during write\n"));
  printf("write ok\n"); 
}


#if 0
static void ram_test()
{
  int r;
  uint8_t D;
  char buf[8];

  int i;
  i= 0;
  buf[i++]= 0xde;
  buf[i++]= 0xad;
  buf[i++]= 0xbe;
  buf[i++]= 0xef;
  buf[i++]= 0xba;
  buf[i++]= 0xad;
  buf[i++]= 0xbe;
  buf[i++]= 0xef;

  printf("RAM TEST\n");
 

 printf("starting to read\n");
  D= 0;
  r= usb_write(I2C_fill, 0, 0, &D, 1);
  if (r<0) exit_print_libusb_error("FILL", r);

  r= usb_read(I2C_write_read, DS1307, 8,buf,1);
  if (r<0) exit_print_libusb_error("SENDING WRITE-READ", r);
  if (buf[0]&I2C_error) exit(fprintf(stderr,"I2C error during write-read\n"));

  r= usb_read(I2C_get, 0, 8, data, 8);
  if (r<0) exit_print_libusb_error("READ", r);
printf("read ok: %d\n",r);
pbin_data();
  int i;
  for(i=1;i<=5;i++) data[i] &= ~(1<<7); // clear all zero bits
pbin_data();

  r= usb_write(I2C_fill, 0, 0, data, 8);
  if (r<0) exit_print_libusb_error("FILL", r);
  r= usb_read(I2C_write, DS1307,8, buf, 1);
  if (r<0) exit_print_libusb_error("WRITE", r);
  if (buf[0]&I2C_error) exit(fprintf(stderr,"I2C error during write\n"));
  printf("write ok\n"); 
}
#endif

int main(int argc,char **argv)
{
  usb_init();
  usb_open();

  read_data();
  prt_data();
  if (get_data())
  {
    prt_data();
    write_data();
    read_data();
    prt_data();
  }
  usb_exit(0);
  return 0;
  

}

