#include <stdint.h>
#include <string.h>
#include "tea5767.h"

static uint8_t encode_bits(uint8_t orig, uint8_t Hbit, uint8_t Lbit, uint8_t V)
{
  uint8_t mask;
  uint8_t i;

  mask= 0xff;
  for(i=Lbit;i<=Hbit;i++)
   mask &= ~(1<<i);

  return (orig & mask) | ((V <<Lbit) & ~mask);
}

static void set_field
  (uint8_t *T, uint8_t V, uint8_t byte, uint8_t Hbit, uint8_t Lbit)
{
  T[byte]= encode_bits(T[byte], Hbit, Lbit, V);
}
                                         //             byte start end
void tea5767_Wmute  (uint8_t *T, int V) { set_field(T,V,  0,   7,   7 ); }
void tea5767_Wsm    (uint8_t *T, int V) { set_field(T,V,  0,   6,   6 ); }
void tea5767_Wpll_hi(uint8_t *T, int V) { set_field(T,V,  0,   5,   0 ); }
void tea5767_Wpll_lo(uint8_t *T, int V) { set_field(T,V,  1,   7,   0 ); }
void tea5767_Wsud   (uint8_t *T, int V) { set_field(T,V,  2,   7,   7 ); }
void tea5767_Wssl   (uint8_t *T, int V) { set_field(T,V,  2,   6,   5 ); }
void tea5767_Whlsi  (uint8_t *T, int V) { set_field(T,V,  2,   4,   4 ); }
void tea5767_Wms    (uint8_t *T, int V) { set_field(T,V,  2,   3,   3 ); }
void tea5767_Wmr    (uint8_t *T, int V) { set_field(T,V,  2,   2,   2 ); }
void tea5767_Wml    (uint8_t *T, int V) { set_field(T,V,  2,   1,   1 ); }
void tea5767_Wswp1  (uint8_t *T, int V) { set_field(T,V,  2,   0,   0 ); }
void tea5767_Wswp2  (uint8_t *T, int V) { set_field(T,V,  3,   7,   7 ); }
void tea5767_Wstby  (uint8_t *T, int V) { set_field(T,V,  3,   6,   6 ); }
void tea5767_Wbl    (uint8_t *T, int V) { set_field(T,V,  3,   5,   5 ); }
void tea5767_Wxtal  (uint8_t *T, int V) { set_field(T,V,  3,   4,   4 ); }
void tea5767_Wsmute (uint8_t *T, int V) { set_field(T,V,  3,   3,   3 ); }
void tea5767_Whcc   (uint8_t *T, int V) { set_field(T,V,  3,   2,   2 ); }
void tea5767_Wsnc   (uint8_t *T, int V) { set_field(T,V,  3,   1,   1 ); }
void tea5767_Wsi    (uint8_t *T, int V) { set_field(T,V,  3,   0,   0 ); }
void tea5767_Wpllref(uint8_t *T, int V) { set_field(T,V,  4,   7,   7 ); }
void tea5767_Wdtc   (uint8_t *T, int V) { set_field(T,V,  4,   6,   6 ); }

void tea5767_Wdefaults(uint8_t *T)
{
  memset(T, 0, 5);
  tea5767_Wmute(T,0);
  tea5767_Wsm(T,0);
  tea5767_Wsud(T,0);
  tea5767_Wssl(T, 1);
  tea5767_Wsnc(T, 1);
  tea5767_Whlsi(T,1);
  tea5767_Wpll(T, 12234); // 100Mhz with high side injection
   // table 16 : 32768 Hz.
  tea5767_Wxtal(T, 1);
  tea5767_Wpllref(T, 0);
}

void tea5767_Wpll(uint8_t *T,uint16_t pll)
{
  tea5767_Wpll_hi(T, pll>>8);
  tea5767_Wpll_lo(T, pll&0xff);
}

#define Fif  225e3

 // frequencies are integers in the form XXXy which correspond to XXX.y Mhz.
 // page 30 of datasheet.  
double tea5767_pll2freq(uint16_t pll)
{
  double F;
  if (pll<10708) return  87.5;
  if (pll>13211) return 108.0;
  F= pll * 32768.0 / 4 - Fif;
  
  return F / 1e6;
}

uint16_t tea5767_freq2pll(double frq)
{
  double F;
  if (frq<87.5 || frq>108.0) return 12234;
  F= frq * 1e6;
  return  4*(F+Fif)/32768.0;
}


static uint8_t decode_bits(uint8_t orig, uint8_t Hbit, uint8_t Lbit)
{
  uint8_t mask;
  uint8_t i;
  mask= 0;
  for(i=Lbit;i<=Hbit;i++)
   mask |= (1<<i);
  return (orig&mask)>>Lbit;
}

static uint8_t get_field(uint8_t *T, uint8_t byte, uint8_t Hbit, uint8_t Lbit)
{
  return decode_bits(T[byte], Hbit, Lbit);
}

uint8_t tea5767_Rrf     (uint8_t *T) { return get_field(T, 0, 7, 7 ); }
uint8_t tea5767_Rblf    (uint8_t *T) { return get_field(T, 0, 6, 6 ); }
uint8_t tea5767_Rpll_hi (uint8_t *T) { return get_field(T, 0, 5, 0 ); }
uint8_t tea5767_Rpll_lo (uint8_t *T) { return get_field(T, 1, 7, 0 ); }
uint8_t tea5767_Rstereo (uint8_t *T) { return get_field(T, 2, 7, 7 ); }
uint8_t tea5767_Rif     (uint8_t *T) { return get_field(T, 2, 6, 0 ); }
uint8_t tea5767_Rlev    (uint8_t *T) { return get_field(T, 3, 7, 4 ); }
uint8_t tea5767_Rci     (uint8_t *T) { return get_field(T, 3, 3, 1 ); }

uint16_t tea5767_Rpll   (uint8_t *T) 
{
  uint16_t H,L;
  H= tea5767_Rpll_hi(T);
  L= tea5767_Rpll_lo(T);
  return (H<<8) | L;
}

