

#define I2C_TWBR \
  ((F_CPU/F_I2C-16) <= 511 ?  (F_CPU/F_I2C-16)/2 :(        \
  (F_CPU/F_I2C-16)/4 <= 511 ? (F_CPU/F_I2C-16)/4/2 :(      \
  (F_CPU/F_I2C-16)/16 <= 511 ? (F_CPU/F_I2C-16)/16/2 :(    \
  (F_CPU/F_I2C-16)/64 <= 511 ? (F_CPU/F_I2C-16)/64/2 :     \
    255 ))))

#define I2C_TWPS \
  ((F_CPU/F_I2C-16) <= 511 ?  0 :(     \
  (F_CPU/F_I2C-16)/4 <= 511 ? 1 :(     \
  (F_CPU/F_I2C-16)/16 <= 511 ? 2 :(    \
  (F_CPU/F_I2C-16)/64 <= 511 ? 3 :     \
    3 ))))
  

void i2c_init()
{
  TWBR= I2C_TWBR;
  TWSR &= ~3;
  TWSR |= I2C_TWPS;
}

void i2c_close()
{
   TWCR&=(~(1<<TWEN));
}

void i2c_start()
{
   TWCR=(1<<TWINT)|(1<<TWEN)|(1<<TWSTA);
   while(!(TWCR & (1<<TWINT)));
}

void i2c_stop()
{
   TWCR=(1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
   while(TWCR & (1<<TWSTO));
}

uint8_t i2c_write_byte(uint8_t data)
{
   uint8_t st;
   TWDR=data;	
   TWCR=(1<<TWEN)|(1<<TWINT);
   while(!(TWCR & (1<<TWINT)));
   st= TWSR & 0xf8;
   if (st==0x18 || st==0x28 || st==0x40)
     return 0;
   return 1;
}

uint8_t i2c_read_byte(uint8_t *data,uint8_t ack)
{
   uint8_t st;
   if(ack) TWCR|=(1<<TWEA);
   else    TWCR&=(~(1<<TWEA));
   TWCR|=(1<<TWINT);
   while(!(TWCR & (1<<TWINT)));

   st= TWSR & 0xf8;
   *data=TWDR;
   if (st==0x40 || st==0x08 || st==0x10 || st==0x58 || st==0x50)
      return 0;
   return 1;
}

uint8_t i2c_read(uint8_t addr, uint8_t len)
{
  byte_t i;
  i2c_start();
  if (i2c_write_byte((addr<<1)|1)) goto fail;
  if (len>32) len= 32;
  i2cblen= 0;
  for(i=0;i<len;i++)
  {
    if (i2c_read_byte(i2cbuffer+i2cblen, i!=len-1)) goto fail;
    i2cblen++;
  }
  i2c_stop();
  return I2C_ok | i;
  
fail:
  i2c_stop();
  return I2C_error;
}

uint8_t i2c_write_nostop(uint8_t addr)
{
  byte_t i;
  i2c_start();
  if (i2c_write_byte((addr<<1))) return I2C_error;
  for(i=0;i<i2cblen;i++)
   if (i2c_write_byte(i2cbuffer[i])) return I2C_error;
  return I2C_ok | i;
}

uint8_t i2c_write_read(uint8_t addr, uint8_t len)
{
  uint8_t R;
  R= i2c_write_nostop(addr);
  if (R & I2C_error) goto fail;

  return i2c_read(addr,len);
fail:
  i2c_stop();
  return I2C_error;
}

uint8_t i2c_write(uint8_t addr)
{
  uint8_t R;
  R= i2c_write_nostop(addr);
  i2c_stop();
  return R;
}
