#include <stdint.h>
#include "usb.h"
#include "atmega8.h"
#include "../firmware/usbi2c.h"

static int portop(int reg, int bit, int V)
{
  int op;
  int r;
  if (bit<0 || bit>7) return ;
  if (reg<0 || reg>5) return ;
  if (V<0   || V>1)   return ;
  op= V*48 + reg*8 + bit;

  r= usb_simple(PORTOP, op, 0);
  if (r<0) { print_libusb_error("PORT", r); return 1; }
  return 0;
}

static int pinop(int reg)
{
  uint8_t data[1];
  int r;
  if (reg<0 || reg>2) return -1;
  r= usb_read(PINOP, reg, 0, data, 1);
  if (r<0) { print_libusb_error("PORT", r); return -1; }
  return data[0];
}

int atmega8_portB(int bit,int V) { return portop(0, bit, V); }
int atmega8_ddrB(int bit,int V)  { return portop(1, bit, V); }
int atmega8_portC(int bit,int V) { return portop(2, bit, V); }
int atmega8_ddrC(int bit,int V)  { return portop(3, bit, V); }
int atmega8_portD(int bit,int V) { return portop(4, bit, V); }
int atmega8_ddrD(int bit,int V)  { return portop(5, bit, V); }

int atmega8_pinB() { return pinop(0); }
int atmega8_pinC() { return pinop(1); }
int atmega8_pinD() { return pinop(2); }





atmega8_t atmega8;

void atmega8_ports_init()
{
  int reg, bit;

  reg= 0;
  bit= 0;
  atmega8.port.B0= reg*8 + bit++;
  atmega8.port.B1= reg*8 + bit++;
  atmega8.port.B2= reg*8 + bit++;
  atmega8.port.B3= reg*8 + bit++;
  atmega8.port.B4= reg*8 + bit++;
  atmega8.port.B5= reg*8 + bit++;
  atmega8.port.B6= reg*8 + bit++;
  atmega8.port.B7= reg*8 + bit++;

  reg= 1;
  bit= 0;
  atmega8.ddr.B0= reg*8 + bit++;
  atmega8.ddr.B1= reg*8 + bit++;
  atmega8.ddr.B2= reg*8 + bit++;
  atmega8.ddr.B3= reg*8 + bit++;
  atmega8.ddr.B4= reg*8 + bit++;
  atmega8.ddr.B5= reg*8 + bit++;
  atmega8.ddr.B6= reg*8 + bit++;
  atmega8.ddr.B7= reg*8 + bit++;

  reg= 2;
  bit= 0;
  atmega8.port.C0= reg*8 + bit++;
  atmega8.port.C1= reg*8 + bit++;
  atmega8.port.C2= reg*8 + bit++;
  atmega8.port.C3= reg*8 + bit++;
  atmega8.port.C4= reg*8 + bit++;
  atmega8.port.C5= reg*8 + bit++;
  atmega8.port.C6= reg*8 + bit++;
  

  reg= 3;
  bit= 0;
  atmega8.ddr.C0= reg*8 + bit++;
  atmega8.ddr.C1= reg*8 + bit++;
  atmega8.ddr.C2= reg*8 + bit++;
  atmega8.ddr.C3= reg*8 + bit++;
  atmega8.ddr.C4= reg*8 + bit++;
  atmega8.ddr.C5= reg*8 + bit++;
  atmega8.ddr.C6= reg*8 + bit++;
 
  reg= 4;
  bit= 0;
  atmega8.port.D0= reg*8 + bit++;
  atmega8.port.D1= reg*8 + bit++;
  atmega8.port.D2= reg*8 + bit++;
  atmega8.port.D3= reg*8 + bit++;
  atmega8.port.D4= reg*8 + bit++;
  atmega8.port.D5= reg*8 + bit++;
  atmega8.port.D6= reg*8 + bit++;
  atmega8.port.D7= reg*8 + bit++;

  reg= 5;
  bit= 0;
  atmega8.ddr.D0= reg*8 + bit++;
  atmega8.ddr.D1= reg*8 + bit++;
  atmega8.ddr.D2= reg*8 + bit++;
  atmega8.ddr.D3= reg*8 + bit++;
  atmega8.ddr.D4= reg*8 + bit++;
  atmega8.ddr.D5= reg*8 + bit++;
  atmega8.ddr.D6= reg*8 + bit++;
  atmega8.ddr.D7= reg*8 + bit++;

  atmega8.delay= (1<<7);
}

portops_t* Aportop(portops_t *P, int p, int v)
{
  if (P->len==255) return ;
  if (p==atmega8.delay) v= p+v;
                  else  v= 48*v+p;
  P->data[P->len++]= v;
}

void run_portops(portops_t *P)
{
  usb_write(PORTOP_MULTI,0,0,P->data,P->len);
}


