#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include "../firmware/usbi2c.h"
#include "usb.h"
#include "tea5767.h"
#include "atmega8.h"

#ifdef testing
#define DS1800_DATA  C0
#define DS1800_CLK   C1
#define DS1800_RESET C2
#else
#define DS1800_DATA  D0
#define DS1800_CLK   D1
#define DS1800_RESET D3
#endif

static int current_out;

static int tea5767_command(uint8_t *data)
{
  int r,i;
  r= usb_write(I2C_fill, 0,0, data, 5);
  if (r<0) { print_libusb_error("FILL:", r); return 1; }
  r= usb_read(I2C_write, TEA5767_I2C_ADDR, 0, data, 1);
  if (r<0) { print_libusb_error("WRITE:", r); return 1; }
  if (data[0]&I2C_error)
    { fprintf(stderr,"I2C error during write\n"); return 1;}

  for(i=0;i<5;i++)
  {
    usleep(1e4);

    r= usb_read(I2C_read, TEA5767_I2C_ADDR, 5, data, 1);
    if (r<0) { print_libusb_error("READ:", r); return 1; }
    if (data[0]&I2C_error) 
        { fprintf(stderr,"I2C error during read\n"); return 1; }

    r= usb_read(I2C_get, 0, 5, data, 5);
    if (r<0) { print_libusb_error("GET:", r); return 1; }
    if (tea5767_Rrf(data)) break;
  }
  if (tea5767_Rrf(data)) 
  {
    fprintf(stderr,"radio ready with pll= %x\n", tea5767_Rpll(data));
  }
  else
  {
    fprintf(stderr,"radio not ready on time\n");
  }
  return 0;
}


static void tea5767_init()
{
  uint8_t data[5];
  tea5767_Wdefaults(data);
  tea5767_Wsnc(data,0);

  tea5767_command(data);
}

void tea5767_setfreq(double f)
{
  uint8_t data[5];
  tea5767_Wdefaults(data);
  tea5767_Wsnc(data,0);
  tea5767_Wpll(data,tea5767_freq2pll(f));
  tea5767_command(data);
}

void soundio_init()
{
     // input switch
  atmega8_ddrD(6,1); atmega8_portD(6,0);
  atmega8_ddrD(7,1); atmega8_portD(7,0);
  atmega8_ddrB(0,1); atmega8_portB(0,0);

    // output switch
  atmega8_ddrB(3,1); atmega8_portB(3,0);
  atmega8_ddrB(4,1); atmega8_portB(4,0);
  atmega8_ddrB(5,1); atmega8_portB(5,0);

   // btooth control relay
  atmega8_ddrB(1,1); atmega8_portB(1,0);

  portops_t P;
  P.len= 0;
  Aportop(&P, atmega8.ddr.DS1800_RESET, 1);
  Aportop(&P, atmega8.port.DS1800_RESET, 0);
  Aportop(&P, atmega8.ddr.DS1800_DATA, 1);
  Aportop(&P, atmega8.port.DS1800_DATA, 0);
  Aportop(&P, atmega8.ddr.DS1800_CLK, 1);
  Aportop(&P, atmega8.port.DS1800_CLK, 0);
  run_portops(&P);
}

void toggle_btooth()
{
  atmega8_portB(1,1);
  sleep(4);
  atmega8_portB(1,0);
}

void select_input(int inp)
{
  int A,B,C;
  switch(inp)
  {
  case 0: A= 0; B= 0; C= 0; break;
  case 1: A= 1; B= 1; C= 0; break;
  default: A=1; B= 0; C= 1; break;
  }
  atmega8_portD(6,C);
  atmega8_portD(7,A);
  atmega8_portB(0,B);
}

void select_output(int out)
{
  int A,B,C;

  if (out==current_out) return ;

  switch(out)
  {
   // hah, I wired it incorrectly.
  case 0: A= 0; B= 0; C= 0; break;
  case 1: A= 1; B= 0; C= 1; break;
  default: A=1; B= 1; C= 0; break;
  }
  atmega8_portB(3,C);
  atmega8_portB(4,A);
  atmega8_portB(5,B);

  if (out==1 || current_out==1) toggle_btooth();
  current_out= out;
}

static void write_volume_data(uint16_t v)
{
  portops_t P;
  int i;
  P.len= 0;
  Aportop(&P, atmega8.port.DS1800_RESET, 1);
  Aportop(&P, atmega8.delay, 20);
  for(i=0;i<16;i++)
  {
     Aportop(&P, atmega8.port.DS1800_DATA, v&1); // data
     Aportop(&P, atmega8.delay, 1);
     Aportop(&P, atmega8.port.DS1800_CLK, 1);   // clk
     Aportop(&P, atmega8.delay, 1);
     Aportop(&P, atmega8.port.DS1800_CLK, 0);   // clk
     Aportop(&P, atmega8.delay, 1);
     v= v>>1;
  }
  Aportop(&P, atmega8.delay,5);
  Aportop(&P, atmega8.port.DS1800_RESET, 0);
  Aportop(&P, atmega8.port.DS1800_DATA, 0);
  run_portops(&P);
}

void set_volume(uint8_t v)
{
  write_volume_data( (v<<8) | v);
}



int main(int argc,char **argv)
{
  int i,r;
  usb_init();
  usb_open();

  atmega8_ports_init();

  soundio_init();
  tea5767_init();

  while(1)
  {
 char g[200];
   if (!fgets(g,sizeof(g),stdin)) continue;
   switch(g[0])
   {
   case 'f': tea5767_setfreq(strtod(g+1,NULL)); break;
   case 'i': select_input(atoi(g+1)); break;
   case 'o': select_output(atoi(g+1)); break;
   case 't': toggle_btooth(); break;
   case 'v': set_volume(atoi(g+1)); break;
   case 'q': goto done;
   }
  }
done:
  usb_exit(0);
  return 0;
}

