#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include "libusb.h"
#include "usb.h"

#define VENDOR_ID 0x6666
#define PROD_ID   0x1291 // see ../firmware/usbtiny.h

static libusb_device_handle *devh;

void print_libusb_error(char *op, int code)
{
   fprintf(stderr,"%s: libusb error (%d) %s\n", op, code,
                     libusb_error_name(code)); 
}

int usb_exit(int r)
{
   if (devh)
   {
     libusb_release_interface(devh, 0);
     libusb_attach_kernel_driver(devh,0);
     libusb_close(devh);
   }
   libusb_exit(NULL);
   return r;
}


void exit_print_libusb_error(char *op,int code)
{
   print_libusb_error(op,code);
   exit(usb_exit(1));
}

void usb_exipif(char *op,int R)
{
  if (R<0) exit_print_libusb_error(op, R);
}

void usb_init()
{
   int r;
   const struct libusb_version* ver;

   r= libusb_init(NULL);
   if (r) exit_print_libusb_error("init-lib", r);
   libusb_set_debug(NULL,3); // MAX level.
   ver= libusb_get_version();
   printf("Initialized libusb-%d.%d.%d.%d%s\n",
         ver->major, ver->minor, ver->micro, ver->nano, ver->rc);
}

void usb_open()
{
   devh= libusb_open_device_with_vid_pid(0, VENDOR_ID, PROD_ID);
   if (!devh)
     exit(fprintf(stderr,"can't open device %04x:%04x\n",
                          VENDOR_ID, PROD_ID));
  libusb_detach_kernel_driver(devh,0);
  libusb_claim_interface(devh, 0);
  printf("Opened device.\n");
  // libusb_reset_device(devh);
}

void usb_reopen()
{
   int R;
   R= libusb_reset_device(devh);
   if (R==LIBUSB_ERROR_NOT_FOUND)
   {
     printf("Reset unsuccesful, trying reopen.\n");
     libusb_close(devh);
     msleep(500);
     usb_open();
   }
}

void msleep(uint32_t milliseconds)
{
  usleep(milliseconds * 1000);
}


int usb_write(int cmd, int arg1, int arg2, void *buf, int len)
{
  return libusb_control_transfer(devh,
            LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR,
		cmd, arg1, arg2, buf, len, 1000);
}

int usb_simple(int cmd, int arg1, int arg2)
{
  return libusb_control_transfer(devh, 
            LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR,
              cmd, arg1, arg2, 0, 0, 500);
}

int usb_read(int cmd, int arg1, int arg2, void *buf, int len)
{
  return libusb_control_transfer(devh,
            LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR,
		cmd, arg1, arg2, buf, len, 1000);
}

