#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "usb.h"
#include "gfx.h"
#include "func.h"
#include "mp3iface.h"
#include "list_dir.h"
#include "intstack.h"

static int fou[4];

static intstack_t *pages;

static char *home_dir;

static void setfou(int K)
{
  int i;
  for(i=0;i<4;i++) { fou[3-i]= K%10; K/= 10; }
}

static int getfou()
{
  int i;
  int V= 0;
  for(i=0;i<4;i++) { V= V*10+fou[i]; }
  return V;
}

static void freqset(int delta)
{
  int V;
  char B[10];
  char *C;

  V= getfou() + delta;
  if (V>1080) V= 1080;
  else if (V<875) V= 875;

  if (V!=getfou()) setfou(V);

  sprintf(B,"%d",V/10);
  C= B+strlen(B);
  sprintf(C,".%d",V%10); 
  gfx_set_text(Wfreq, B);
  tea5767_setfreq(V/10.0);
}

int mp3_ok;

typedef struct
{
  char *disp;
  char *file;
  int type;
  int hilight;
} list_entry_t;

typedef struct
{
  int n_entries,s_entries;
  list_entry_t *entries;
  int offset;
  int cursor;
} dlist_t;

#define WSIZE 13
static int dlistlab[WSIZE] = 
{ PLlab1, PLlab2, PLlab3, PLlab4, PLlab5, PLlab6, PLlab7,
  PLlab8, PLlab9, PLlab10, PLlab11, PLlab12, PLlab13};

static dlist_t *current_list;

static void dlist_update_item(dlist_t *DD,int idx)
{
  int P;
  int fg,bg;
  if (idx<DD->offset || idx>=DD->offset+WSIZE) return ;
  P= idx%WSIZE;
  if (DD->entries[idx].hilight) { fg= 0xffFFff; bg= 0xFF0000; }
  else if (DD->cursor==idx) { fg= 0xFFffFF; bg= 0x00ffff; }
  else { fg= 0x00; bg= 0xFFffFF; }

  gfx_set_fgbg(dlistlab[P], fg, bg);
  gfx_set_text(dlistlab[P], DD->entries[idx].disp);
}

static void dlist_append(dlist_t *DD,char *f,char *d,int type,int upd)
{
  int N;
  if (DD->n_entries==DD->s_entries)
  {
    DD->s_entries+= DD->s_entries/2+10;
    DD->entries= realloc(DD->entries, 
                            DD->s_entries*sizeof(list_entry_t));
  }
  N= DD->n_entries++;
  DD->entries[N].file= strdup(f);
  DD->entries[N].disp= strdup(d);
  DD->entries[N].type= type;
  DD->entries[N].hilight= 0;
  if (upd)
    dlist_update_item(DD,N);
}
static void dlist_redisplay(dlist_t *DD)
{
  int i,n;
  for(i=0;i<WSIZE;i++)
  {
    n= i+DD->offset;
    if (n>=DD->n_entries) break;
    dlist_update_item(DD,n);
  }
  for(;i<WSIZE;i++)
  {
    gfx_set_fgbg(dlistlab[i], 0x00, 0xFFffFF);
    gfx_set_text(dlistlab[i], "");
  }
}

static void dlist_clear(dlist_t *DD)
{
  int i;
  for(i=0;i<DD->n_entries;i++)
  {
    free(DD->entries[i].disp);
    free(DD->entries[i].file);
  }
  if (DD->entries) free(DD->entries);
  DD->entries= NULL;
  DD->n_entries= DD->s_entries= 0;
  DD->offset= 0;
  DD->cursor= 0;
  if (DD==current_list) dlist_redisplay(DD);
}

static void dlist_set_cursor(dlist_t *DD,int c)
{
  int oc;
  if (DD->cursor==c || c<0 || c>=DD->n_entries) return ;
  oc= DD->cursor;
  DD->cursor= c;
  if (c<DD->offset || c>=DD->offset+WSIZE)
  {
    DD->offset= c-c%WSIZE;
    dlist_redisplay(DD);
    return ;
  }
  if (oc>=DD->offset && oc<DD->offset+WSIZE)
  {
    dlist_update_item(DD,oc);
  }
  dlist_update_item(DD,c);
}

static void dlist_cursor_mid(dlist_t *DD)
{
  int mid;
  mid= DD->offset + WSIZE/2;
  if (mid>=DD->n_entries) mid= (DD->offset+DD->n_entries)/2;
  if (mid>=DD->n_entries) return ;
  dlist_set_cursor(DD,mid);
}

static void dlist_page_down(dlist_t *DD)
{
  if (DD->offset+WSIZE>=DD->n_entries) return ;
  DD->offset+= WSIZE;
  DD->cursor= DD->offset;
  dlist_redisplay(DD);
}

static void dlist_page_up(dlist_t *DD)
{
  int ff;
  ff= DD->offset-WSIZE;
  if (ff<0) ff=0;
  if (DD->offset==ff) return ;
  DD->offset= ff;
  DD->cursor= DD->offset;
  dlist_redisplay(DD);
}

static dlist_t *flist, *tlist;
char *current_dir;

static char *combine_fn(char *dir,char *fn)
{
  char *R;
  R= malloc(strlen(dir)+strlen(fn)+2);
  sprintf(R,"%s/%s",dir,fn);
  return R;
}

static void playlist_add(char *dir,char *fn)
{
  char *file,*disp;
  int L;
  if (!tlist) tlist= calloc(1,sizeof(*tlist));
  file= combine_fn(dir,fn);
  disp= strdup(fn);
  L= strlen(disp);
  while(L>0 && disp[L]!='.') L--;
  if (L) disp[L]= 0;
  dlist_append(tlist, file, disp, 0, 0);
  free(file);
  free(disp);
}

static void populate_file_list(dlist_t *LL,char *dirname)
{
  char **files;
  int i;
  dlist_clear(LL);
  files= list_dir(dirname);
  if (current_dir)  free(current_dir);  
  current_dir= strdup(dirname);
  for(i=0;files[i];i++)
  {
    if (files[i][0]=='D') files[i][0]='<';
             else         files[i][0]=' ';
    dlist_append(LL, files[i]+1, files[i], files[i][0]=='<',1);
  } 
  free(files);
}

static char *find_next_track(char *tn)
{
  int i;
  if (!tlist) return NULL;
  for(i=0;i<tlist->n_entries;i++)
   if (!strcmp(tn,tlist->entries[i].file))
   {
     if (i==tlist->n_entries-1) return NULL;
     return tlist->entries[i+1].file;
   }
  if (tlist->n_entries)
    return tlist->entries[0].file;
  return NULL;
}

static char *find_prev_track(char *tn)
{
  int i;
  if (!tlist) return NULL;
  for(i=0;i<tlist->n_entries;i++)
   if (!strcmp(tn,tlist->entries[i].file))
   {
     if (i==0) return NULL;
     return tlist->entries[i-1].file;
   }
  if (tlist->n_entries)
    return tlist->entries[0].file;
  return NULL;
}

static void tlist_set_selected(char *file)
{
  int i;
  int c;
  if (!tlist) return ;
  c= 0;
  for(i=0;i<tlist->n_entries && c<2;i++)
  {
    int hi,sa;
    hi= tlist->entries[i].hilight ? 1: 0;
    sa= strcmp(file, tlist->entries[i].file) ? 0 : 1;
    switch( (hi<<1) | sa )
    {
      case (0<<1) | 0: break;
      case (1<<1) | 0: 
          c++;
          tlist->entries[i].hilight= 0;
          if (current_list==tlist) dlist_update_item(tlist, i);
          break;
      case (0<<1) | 1:
          c++;
          tlist->entries[i].hilight= 1;
          if (i<tlist->offset || i>=tlist->offset+WSIZE)
          {
            tlist->offset= i - i%WSIZE;
            if (current_list==tlist) 
              dlist_redisplay(tlist);
          }
          else
          {
             if (current_list==tlist) 
               dlist_update_item(tlist, i);
          }
          break;
      case (1<<1) | 1:
          return ;
    }
  }
}

static char *current_track;

int main()
{
  gfx_event_t event;

  switch_init();

  gfx_init(NULL, 480, 800,"Player" );
  gfx_main_screen();

  int mpfd;
  mpfd= mp3_open_player("test.fifo","test.fifor");
  gfx_monitor_file(mpfd,1,1,0,0);

  home_dir= getenv("AUS_HOME");
  if (!home_dir) home_dir= "/home/dodo/Music";
  home_dir= strdup(home_dir);

  setfou(1000);

  int paused= 1;
  int track_end;
  double dummy;


  while(1)
  {
   if (!gfx_event(NULL,&event)) continue;
   fprintf(stderr,"got an event: %d\r\n",event.type);
   switch(event.type)
    {
    case GFXEV_QUIT: goto done;
    case Wplayer:
       select_input(2);
       gfx_set_selected(MAIN_PANEL,1);
       if (!paused) mp3_play();
       break;
    case Wplay:
       if (current_track)
       {
         mp3_play(); 
         paused= 0;
       }
       else if (tlist && tlist->n_entries)
       {
         current_track= strdup(tlist->entries[0].file);
         tlist_set_selected(current_track);
         mp3_load_file(current_track, &dummy);
         mp3_play();
         paused= 0;
       }
      break;
    case Wpause: mp3_pause(); paused= 1; break;
    case Wexternal: select_input(0); mp3_pause(); break;
    case Wradio: 
       gfx_set_selected(MAIN_PANEL,0);
       select_input(1); 
       mp3_pause();
       break;
    case GFXEV_READY_READ:
        track_end= 0;
        mp3_read(&track_end);
        if (track_end)
        {
          char *nex;
          nex= find_next_track(current_track);
          if (!nex) { paused= 1; continue; }
          tlist_set_selected(nex);
          if (current_track) free(current_track);
          current_track= strdup(nex);
          mp3_load_file(current_track,&dummy);
          mp3_play();
          paused= 0;
        }
        break;
    case Wnexttrack:
        { char *nex;
          if (!current_track)
          {
           if (!tlist || !tlist->n_entries) continue ; 
           current_track= strdup(tlist->entries[0].file);
           tlist_set_selected(current_track);
           mp3_load_file(current_track,&dummy);
           if (!paused) mp3_play();
          } else {
            nex= find_next_track(current_track);
            if (!nex) { paused= 1; continue; }
            tlist_set_selected(nex);
            if (current_track) free(current_track);
            current_track= strdup(nex);
            mp3_load_file(current_track,&dummy);
            if (!paused) mp3_play();
          }
        }
          break;
    case Wprevtrack:
        { char *nex;
          if (!current_track)
          {
           if (!tlist || !tlist->n_entries) continue ; 
           current_track= strdup(tlist->entries[0].file);
           tlist_set_selected(current_track);
           mp3_load_file(current_track,&dummy);
           if (!paused) mp3_play();
          } else {
            nex= find_prev_track(current_track);
            if (!nex) { paused= 1; continue; }
            tlist_set_selected(nex);
            if (current_track) free(current_track);
            current_track= strdup(nex);
            mp3_load_file(current_track,&dummy);
            if (!paused) mp3_play();
          }
        }
          break;
    case Wjump:
       {
        char *T;
        if (!tlist || !tlist->n_entries || tlist->cursor<0 || 
             tlist->cursor>=tlist->n_entries) continue;
        T= tlist->entries[tlist->cursor].file;
        if (current_track) free(current_track);
        current_track= strdup(T);
        tlist_set_selected(current_track);
        mp3_load_file(current_track,&dummy);
        if (!paused) mp3_play();
        break;
       }
    case Wspeaker: select_output(0); break;
    case Wheadphones: select_output(2); break;
    case Wbluetooth: select_output(1); break;

    case Wfreqless: freqset(-1); break;
    case Wfreqmore: freqset(+1); break;
    case PLmiddle: 
      if (current_list) dlist_cursor_mid(current_list);
      break;
    case PLup:
      if (current_list) dlist_set_cursor(current_list,current_list->cursor-1);
      break;
    case PLdown:
      if (current_list) dlist_set_cursor(current_list,current_list->cursor+1);
      break;
    case PLpageup:
      if (current_list) dlist_page_up(current_list); break;
    case PLpagedown:
      if (current_list) dlist_page_down(current_list); break;

    case Wfiles: 
      gfx_set_selected(Wplayertoppanel,1);
      if (!flist)
        flist= calloc(1,sizeof(*flist));
      current_list= flist;
      if (!current_dir)
        populate_file_list(flist,home_dir);
      else
        dlist_redisplay(flist);
      break;
    case Waddlist:
      if (current_list==flist)
      { if (flist->cursor<0 || flist->cursor>=flist->n_entries) break;
        list_entry_t *L;
        L= flist->entries+flist->cursor;
        if (L->type)
        {
          char **files;
          char *path;
          int i;
          path= combine_fn(current_dir, L->file);
          files= list_dir(path);
          for(i=0;files[i];i++)
           if (files[i][0]=='F')
             { playlist_add(path, files[i]+1); }
          free(path);
          free(files);
        }
        else
        {
          playlist_add(current_dir, L->file);
        }
      }
      break;
    case Wplaylist:
      gfx_set_selected(Wplayertoppanel,0);
      if (!tlist)
        tlist= calloc(1,sizeof(*tlist));
      current_list= tlist;
      dlist_redisplay(tlist);
      break;
    case Wdirlist:
      { char *path;
        if (!flist || flist->cursor<0 || flist->cursor>=flist->n_entries)
          continue;
        if (!current_dir) continue;
        list_entry_t *L= flist->entries+flist->cursor;
        if (!L->type) continue;
        intstack_push(&pages, flist->offset);
        intstack_push(&pages, flist->cursor);
        path= combine_fn(current_dir,L->file);
        populate_file_list(flist,path); free(path);
        break; }
    case Wdirup:
      { if (!current_dir || !strcmp(current_dir,home_dir)) continue;
        char *K;
        int L;
        int pos,cur;
        K= strdup(current_dir);
        L= strlen(K);
        while(L>0 && K[L]!='/') L--;
        if (!L) { free(K); continue; }
        K[L]= 0; 
        if (intstack_pop(&pages,&cur)) cur= 0;
        if (intstack_pop(&pages,&pos)) pos= 0;
        populate_file_list(flist, K);
        free(K);
        flist->offset= pos;
        flist->cursor= cur;
        if (pos||cur) dlist_redisplay(flist); break; }
    case Whome:
      {
         int dummyint;
         while(!intstack_pop(&pages,&dummyint)) ;
         if (!flist) flist= calloc(1,sizeof(*flist));
         populate_file_list(flist, home_dir);
         break;
      }
    case Wclear:
      dlist_clear(tlist);
      break;
    }
  }
done:
  usb_exit(0);
  return 0;
}
