#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "mp3iface.h"
#include "proc.h"

static int fd=-1;
static int playing_status=1; // 2 = playing
                             // 1 = paused
static int total_samples;
static char cmd[50],arg1[50],arg2[50];

static int ifd= -1;

static void send_cmd(char *cmd,...)
{
  char *a;
  char buf[2000];
  va_list ap;

  if (fd<0) return ; 

  buf[0]= 0;

  a= cmd;
  va_start(ap,cmd);
  while(1)
  {
    strcat(buf, a);
    a= va_arg(ap, char*);
    if (!a) break;
    strcat(buf," ");
  }
  va_end(ap);
  strcat(buf,"\n");
  write(fd,buf,strlen(buf));
  // printf("sending %s",buf);
}

static void recv_cmd()
{
  int tlen;
  char *tgts[4];
  int  tnex[4],tinc[4];
  int T;
  int R;
  char dummy[50];
  unsigned char c;

  tgts[0]= cmd; tgts[1]= arg1; tgts[2]= arg2; tgts[3]= dummy;

  tnex[0]= 1; tnex[1]= 2; tnex[2]= 3; tnex[3]= 3;
  tinc[0]= 1; tinc[1]= 1; tinc[2]= 1; tinc[3]= 0;
  T= 0;
  tlen= 0;

  arg1[0]= 0; 
  arg2[0]= 0; 
  cmd[0]= 0;

  printf("receiving ..");

again:
  if (ifd<0)
  {
 printf("ifd is now %d\n",ifd);
    strcpy(cmd,"@E"); 
    return ;
  }

  while(1)
  {
    if (ifd<0) goto again;
    R= read(ifd,&c,1);
    if (R<=0) {
        int eno= errno;
         printf("read returns %d eno=%d eintr=%d\n",R,eno,EINTR);
        if (eno==EINTR) continue;
        close(ifd); ifd= -1; strcpy(cmd,"@E"); return ;
    }
     printf("%c",c); 
    if (c=='\n') { 
       printf("  Received: %s %s %s\n", cmd,arg1,arg2); 
       return ; 
    }
    if (c<=32 || c>126)
    { if (tlen==0) continue; 
      T= tnex[T]; tlen= 0; }
    if (tlen<sizeof(cmd)-1)
    {
      tgts[T][tlen]= c;
      tgts[T][tlen+1]= 0;
      tlen+= tinc[T];
    }
  }
}

static int cmd_is(char *k)
{
  if (!strcmp(k,cmd)) return 1;
  return 0;
}

static int cmd_p()
{
  if (cmd_is("@P")) { playing_status= atoi(arg1); return 1; }
  return 0;
}

static int cmd_is_error()
{
  return cmd_is("@E");
}


int mp3_open_player(char *ofn,char *ifn)
{
  char *argv[3];
  int id;
  static char *player;
  if (!player) { player= getenv("AUS_MPG123");
                 if (!player) player= "/home/dodo/bin/mpg123";
                 player= strdup(player); }
  argv[0]= player;
  argv[1]= "-R";
  argv[2]= NULL;
  id= proc_start(argv[0],argv,PROC_INP|PROC_OUTP);
  if (id<0) return -1;
  // fd= open(ofn,O_WRONLY);
  // ifd= open(ifn,O_RDONLY);
  // if (fd<0 || ifd<0) return -1;
  ifd= proc_output(id);
  fd= proc_input(id);
  send_cmd("silence",NULL);
  return ifd;
}

int mp3_load_file(char *fn,double *Rdur)
{
  int rate, samples,chan;
  double dur;

  send_cmd("lp",fn,NULL);
  while(1)
  {
   recv_cmd();
   if (cmd_is_error()) return 1;
   if (cmd_p()) break;
  }
  send_cmd("sample",NULL);
  while(1)
  {
   recv_cmd();
   if (cmd_is_error()) return 1;
   if (cmd_is("@SAMPLE")) { samples= atoi(arg2); break; }
  }
#if 0
  send_cmd("format",NULL);
  while(1)
  {
   recv_cmd();
   if (cmd_is_error()) return 1;
   if (cmd_is("@FORMAT")) { rate= atoi(arg1); chan= atoi(arg2); break; }
  }
  *Rdur= samples/((double) rate*chan);
#endif
  *Rdur= 60;
  total_samples= samples;
  return 0; 
}

void mp3_pause()
{
  if (playing_status==1) return ;
  send_cmd("p",NULL);
  while(1)
  {
    recv_cmd();
    if (cmd_is_error()) return ;
    if (cmd_p()) break;
  }
}

void mp3_play()
{
  if (playing_status==2) return ;
  send_cmd("p",NULL);
  while(1)
  {
    recv_cmd();
    if (cmd_is_error()) return ;
    if (cmd_p()) break;
  }
}

int mp3_getpos(double *Rp)
{
  int pos;
  send_cmd("sample",NULL);
  while(1)
  {
   recv_cmd();
   if (cmd_is_error()) return 1;
   if (cmd_is("@SAMPLE")) { pos= atoi(arg1); break; }
  }
  if (total_samples) *Rp= (double) pos / total_samples;
                else *Rp= 0.0;
  return 0;
}

void mp3_close_player()
{
  send_cmd("q");
  close(fd);
  fd= -1;
}

void mp3_read(int *end)
{
  int ps;
  printf("trying to read something\r\n");
  ps= playing_status;
  recv_cmd();
  cmd_p();
  if (playing_status==0 && ps!=0) *end= 1;
  else *end= 0;
}
