#include "config.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "def.h"
#include "usb.h"
#include "usbled.h"

#define LED1 (C,5)
#define LED2 (C,4)


uint8_t usb_in(byte_t* data, byte_t len)
{
   // return the data requested by the usb transfer.
   return 0;
}

void usb_out(byte_t* data, byte_t len)
{
   // use the data specified by the usb transfer
   // here, we have no data so we simply skip it.
}

byte_t usb_setup(byte_t data[8])
{
  uint8_t req;
  uint16_t value;
  // the setup packet is structured as follows:
  // offset size meaning
  //  0      1   request type, describes the endpoint direction etc.
  //  1      1   request code. any value is OK, since the above
  //                           field already specifies VENDOR and
  //                           we're free to use any value.
  //  2      2   value    these two fields are generic fields for
  //  4      2   index    identifying stuff. I just use them as data fields
  //  6      2   length of data to follow in the _out() packets
  req= data[1];
  // if there is data to be returned, we can do that within the data[]
  // array, using up to 8 bytes. If there are more, we should return
  // 0xff and usbtiny will call usb_in().
  switch(req)
  {
  case CMD_LED1:
    value= *(uint16_t*) (data+2);
    if (value) { PORTC |= 16; } else { PORTC &= ~16; }
    return 0;
  case CMD_LED2:
    value= *(uint16_t*) (data+2);
    if (value) { PORTC |= 32; } else { PORTC &= ~32; }
    return 0;
  case CMD_STATUS:
    data[0]= PORTC;
    return 1;
  }
  return 0; 
}


int main()
{
   usb_init();
   PINC= 0;
   PORTC= 0;
   DDRC= 48;

   while(1) usb_poll();
}

